import { Container, ProductSite } from '@sage/x3-master-data-api';
import { Location } from '@sage/x3-stock-data-api';
import * as ui from '@sage/xtrem-ui';
import { Product, ProductLine, StorageObject, TrackingLine } from './utils/types';
export declare class MobileProductionReportingDetails extends ui.Page {
    product: ui.fields.Text;
    localizedDescription: ui.fields.Text;
    site: ui.fields.Text;
    workOrderNumber: ui.fields.Text;
    section: ui.containers.Section;
    block: ui.containers.Block;
    image: ui.fields.Image;
    stockUnit: ui.fields.Text;
    remainingQuantity: ui.fields.Numeric;
    stockStatus: ui.fields.Select;
    suggestedLocation: ui.fields.Reference;
    location: ui.fields.Reference;
    lot: ui.fields.Reference;
    lotText: ui.fields.Text;
    sublot: ui.fields.Text;
    expirationDate: ui.fields.Date;
    serialNumber: ui.fields.Text;
    container: ui.fields.Reference;
    licensePlateNumber: ui.fields.Reference;
    close: ui.fields.Checkbox;
    lotManaged: ui.fields.Numeric;
    sublotManaged: ui.fields.Numeric;
    addQuantity: ui.PageAction;
    nextPage: ui.PageAction;
    initPage(): Promise<void>;
    private getProductSite;
    /**
     * Get stock site from this.$.storage
     * @returns string -  site code
     */
    getStockSite(): string;
    getStorageObject(): StorageObject | undefined;
    getProductLine(): Partial<ProductLine>;
    getTrackingLines(): TrackingLine[];
    showStorageError(): void;
    initTrackingLines(storageObject: any): TrackingLine[];
    saveQuantityTracking(storageObject: StorageObject): void;
    buttonsLogic(displayMsg: boolean): Promise<void>;
    /** Fields Methods */
    calculateQuantity(lines: TrackingLine[], productLine: Partial<ProductLine>): number;
    toggleManageBySerialNumberField(serialNumberManagementMode: string): boolean;
    toggleManageByLocationFields(locationManagementMode: any): void;
    private lotManagement;
    expirationManagement(lotManagement: string, product: string): Promise<void>;
    containerFieldLogic(containerValue: Partial<Container>, locationValue: Partial<Location>): void;
    /**
     * Populate header card
     * @param {Partial<Product>} { code, localizedDescription1 }
     * @param {string} site
     * @memberof MobileProductionReportingDetails
     */
    setHeaderFields({ code, localizedDescription1 }: Partial<Product>, site: string): Promise<void>;
    getStockStatuses(): Promise<string[]>;
    getLocationManagementRules(product: string, stockSite: string): Promise<import("@sage/xtrem-client").ExtractEdges<ProductSite>>;
    getLotManagementMode(): Promise<string>;
    getCategoryLotManagementMode(productCategory: string): Promise<string>;
    private lotExists;
    /**
     * findSerialNumber - search table STOSER for a selected product serial number
     * @param product
     * @param serialNumber
     * @returns boolean
     */
    findSerialNumber(product: any, serialNumber: string): Promise<boolean>;
    getExpirationManagementMode(product: string): Promise<any>;
    /** Get the LOT sequence for a product */
    getLotSequence(): Promise<string>;
    /**
     *  Set the isHidden and isMandatory values for UI
     *  properties affected by a products lot management mode
     *
     * @private
     * @param {string} mode
     * @memberof MobileProductionReportingDetails
     */
    private setLotFieldBehaviour;
    /**
     * Test that the required lot values have been entered as
     * required by the products lot management rules
     * @param {*} mode
     * @return {*}  {Promise<boolean>}
     * @memberof MobileProductionReportingDetails
     */
    lotValuesAreValid(mode: string): Promise<boolean>;
}
//# sourceMappingURL=mobile-production-reporting-details.d.ts.map